@@ -3,12 +3,12 @@
 #     /  \		(C) Copyright 2003 Parliament Hill Computers Ltd.
 #     \  /		All rights reserved.
 #      \/
-#       .		Author: Alain Williams, First written January 2003; updated March 2015
+#       .		Author: Alain Williams, First written January 2003; last update July 2016
 #       .		addw@phcomp.co.uk
 #        .
 #          .
 #
-#	SCCS: @(#)Expression.pm	1.45 04/11/15 00:37:10
+#	SCCS: @(#)Expression.pm	1.47 07/21/16 12:48:37
 #
 # This module is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself. You must preserve this entire copyright
@@ -50,7 +50,7 @@ use vars qw/
 	$Version
 );
 
-our $VERSION = "1.45";
+our $VERSION = "1.47";
 
 # Fundamental to this is a tree of nodes.
 # Nodes are hashes with members:
@@ -62,8 +62,16 @@ our $VERSION = "1.45";
 # val (on const nodes)
 # flow (on flow nodes)
 
+# Within ParseString() there are 2 stacks:
+# @tree (of nodes) - this is what is eventually returned
+#   Terminals (var, const) are pushed here as they are read in
+# @operators all non terminals with JR-precedence > TOS-precedence start off being pushed here.
+#   Where JR-precedence <= TOS-precedence do 'reduce', ie move from @operators to @tree as a tree,
+#   with left/right children coming from @tree and the operator pushed to @tree.
+# It is interesting to print the tree with the Data::Dumper or PrintTree().
 
-# Operator precedence, higher means evaluate first.
+
+# Operator precedence, higher means bind more tightly to operands - ie evaluate first.
 # If precedence values are the same associate to the left.
 # 2 values, depending on if it is the TopOfStack or JustRead operator - [TOS, JR]. See ':=' which right associates.
 # Just binary operators makes life easier as well.
@@ -141,9 +149,10 @@ my %MatchOp = (
 
 my %MatchOpClose = reverse %MatchOp; # Reverse lookup
 
-# Inbuilt functions
+# Inbuilt functions, copied to property Functions
 my %InFuns = map { $_ => 1} qw/ abs aindex count defined int join localtime mktime pop printf push round shift split strftime strlen unshift /;
 # Inbuilt functions that must be given a L value
+# This does not need to be externally visible with our, any ExtraFuncEval that adds to it will cope
 my %InFunLV = map { $_ => 1} qw / defined pop push shift unshift /;
 
 # Escape chars recognised:
@@ -239,6 +248,7 @@ sub ParseString {
 	my @operators = ();		# Operators stacked here until needed
 	my @tree;			# Parsed tree ends up here
 	my $newt;			# New Token
+	my $ln = '';			# Last $newt->{oper}
 
 	my $operlast = 1;		# Operator was last, ie not: var, const, ; ) string flow. Used to idenify monadic operators
 	my $endAlready = 0;
@@ -279,8 +289,8 @@ sub ParseString {
 		elsif($expr =~ /^(['"])/ and $expr =~ s/^($1)([^$1]*)$1//) {
 			$newt->{oper} = 'const';
 			$newt->{val} = $2;
-			# Double quoted: understand some escapes:
-			$newt->{val} =~ s/\\([nrt\\]|x[\da-fA-F]{2})/length($1) == 1 ? $escapes{$1} : (chr hex '0'.$1)/ge if($1 eq '"');
+			# Double quoted, understand some escapes:
+			$newt->{val} =~ s/\\([nrt\\]|x[\da-fA-F]{2}|u\{([\da-fA-F]+)\})/length($1) == 1 ? $escapes{$1} : defined($2) ? (chr hex $2) : (chr hex '0'.$1)/ge if($1 eq '"');
 			$newt->{type} = 'str';
 			$operlast = 0;
 		} elsif($expr =~ s/^}//) {
@@ -301,8 +311,12 @@ sub ParseString {
 				$newt->{monop} = $1;		# Monop flag & for error reporting
 			}
 
-			$operlast = 1 unless($1 eq ')' or $1 eq ']');
-
+			# If we see '()' push the empty list as '()' will just be eliminated - as if never there.
+			if($ln eq '(' && $1 eq ')') {
+				push @tree, {oper => 'var', name => 'EmptyList'};
+			} else {
+				$operlast = 1 unless($1 eq ')' or $1 eq ']');
+			}
 		} # Flow: if/while:
 		elsif($expr =~ s@^(if|while)@@) {
 			$newt->{oper} = 'flow';
@@ -313,9 +327,9 @@ sub ParseString {
 			$newt->{oper} = ';';
 			$operlast = 0;
 		} # Match 'function(', leave '(' in input:
-		elsif($expr =~ s/^([_a-zA-Z][\w]*)\(/(/) {
-			unless($InFuns{$1}) {
-				$self->PrintError("Unknown function '%s'", $1);
+		elsif($expr =~ s/^([_a-z][\w]*)\(/(/i) {
+			unless($self->{Functions}->{$1}) {
+				$self->PrintError("When parsing: found unknown function '%s'", $1);
 				return;
 			}
 			$newt->{oper} = 'func';
@@ -336,6 +350,9 @@ sub ParseString {
 			return pop @tree;
 		}
 
+		# What is new token ?
+		$ln = $newt ? $newt->{oper} : '';
+
 		# Grammatical part
 		# Move what we can from @operators to @tree
 
@@ -439,8 +456,10 @@ sub ParseString {
 					return;
 				}
 
+				# Push $op to @tree, first give it right & left children taken from the top of @tree
 				$op->{right} = pop @tree;
 				unless($monop or $func) {
+					# Monadic operators & functions do not have a 'left' child.
 					$op->{left} = pop @tree;
 				}
 
@@ -504,7 +523,7 @@ sub CheckTreeInt {
 
 	if($tree->{oper} eq 'func') {
 		my $fname = $tree->{fname};
-		if(($fname eq 'pop' or $fname eq 'shift' or $fname eq 'push' or $fname eq 'unshift') and
+		if($InFunLV{$fname} and
 		   (!defined($tree->{right}->{oper}) or (($tree->{right}->{oper} ne 'var' and $tree->{right}->{oper} ne ',') and (!defined($tree->{right}->{left}->{oper}) or $tree->{right}->{left}->{oper} ne 'var')))) {
 			$self->PrintError("First argument to $fname must be a variable");
 			$ok = 0;
@@ -541,7 +560,7 @@ sub PrintTree {
 	}
 
 	print "    " x $dl;
-	print "nod=$nodp [$nodp->{oper}] P $OperPrec{$nodp->{oper}}[0] ";
+	print "nod=$nodp [$nodp->{oper}] P-JR $OperPrec{$nodp->{oper}}[1] ";
 
 	if($nodp->{oper} eq 'var') {
 		print "var($nodp->{name}) \n";
@@ -552,7 +571,7 @@ sub PrintTree {
 		print "    " x $dl;print "Desc L \n";
 		$self->PrintTree($nodp->{left}, $dl);
 
-		print "    " x $dl;print "op '$nodp->{oper}' P $OperPrec{$nodp->{oper}}[0] at $nodp\n";
+		print "    " x $dl;print "op '$nodp->{oper}' P-TOS $OperPrec{$nodp->{oper}}[0] at $nodp\n";
 
 		print "    " x $dl;print "Desc R \n";
 		$self->PrintTree($nodp->{right}, $dl);
@@ -937,7 +956,7 @@ sub FuncValue {
 		return -1;
 	}
 
-	$self->PrintError("Unknown function '$fname'");
+	$self->PrintError("Unknown Function '$fname'");
 
 	return '';
 }
@@ -967,6 +986,7 @@ sub new {
 		ArrayMaxIndex	=>	100,			# Max index of an array
 		StringMaxLength	=>	1000,			# Max string length
 		EnablePrintf	=>	0,			# Enable printf function
+		Functions	=>	{%InFuns},		# Known functions, initialise to builtins
 
 	);
 
@@ -996,42 +1016,6 @@ __END__
 
 Math::Expression - Safely evaluate arithmetic/string expressions
 
-=head1 SYNOPSIS
-
-Shipping cost depends on item price by some arbitrary formula. The VAT amount can also
-vary depending on political edict. Rather than nail these formula into the application code the
-formula are obtained at run time from some configuration source. These formula are probably
-entered by a non technical manager and are thus not to be trusted.
-
-    use Math::Expression;
-    my $ae = new Math::Expression;
-
-# Obtain from a configuration source:
-    my $ShippingFormula = 'Price >= 100 ? Price * 0.1 : (Price >= 50 ? Price * 0.15 : Price * 0.2)';
-    my $VatFormula = 'VatTax := Price * 0.2';
-
-# Price of what you are selling, set the price variable:
-    my $price = 100;
-    $ae->VarSetScalar('Price', $price);
-
-# Obtain VAT & Shipping using the configured formula:
-    my $VatTax = $ae->ParseToScalar($VatFormula);
-    my $Shipping  = $ae->ParseToScalar($ShippingFormula);
-
-    say "Price=$price VatTax=$VatTax Shipping=$Shipping";
-
-# If these will be run many times, parse the formula once:
-
-    my $VatExpr = $ae->Parse($VatFormula);
-    my $ShipExpr = $ae->Parse($ShippingFormula);
-
-# Evaluate it with the current price many times:
-
-    $ae->VarSetScalar('Price', $price);
-    $VatTax = $ae->EvalToScalar($VatExpr);
-    $Shipping = $ae->EvalToScalar($ShipExpr);
-
-
 =head1 DESCRIPTION
 
 Evaluating an expression from an untrusted source can result in security or denial of service attacks.
@@ -1039,7 +1023,8 @@ Sometimes this needs to be done to do what the user wants.
 
 This module solves the problem of evaluating expressions read from sources such as config/...
 files and user web forms without the use of C<eval>.
-String and arithmetic operators are supported, as are: variables, loops, conditions, arrays and functions.
+String and arithmetic operators are supported (as in C/Perl),
+as are: variables, loops, conditions, arrays and be functions (inbuilt & user defined).
 
 The program may set initial values for variables and obtain their values once the expression
 has been evaluated.
@@ -1049,6 +1034,42 @@ variable values.
 Error messages may be via a user provided function.
 This is not designed for high computation use.
 
+=head1 EXAMPLE
+
+Shipping cost depends on item price by some arbitrary formula. The VAT amount can also
+vary depending on political edict. Rather than nail these formula into the application code the
+formula are obtained at run time from some configuration source. These formula are
+entered by a non technical manager and are thus not to be trusted.
+
+    use Math::Expression;
+    my $ArithEnv = new Math::Expression;
+
+# Obtain from a configuration source:
+    my $ShippingFormula = 'Price >= 100 ? Price * 0.1 : (Price >= 50 ? Price * 0.15 : Price * 0.2)';
+    my $VatFormula = 'VatTax := Price * 0.2';
+
+# Price of what you are selling, set the price variable:
+    my $price = 100;
+    $ArithEnv->VarSetScalar('Price', $price);
+
+# Obtain VAT & Shipping using the configured formula:
+    my $VatTax = $ArithEnv->ParseToScalar($VatFormula);
+    my $Shipping  = $ArithEnv->ParseToScalar($ShippingFormula);
+
+    say "Price=$price VatTax=$VatTax Shipping=$Shipping";
+
+# If these will be run many times, parse the formula once:
+
+    my $VatExpr = $ArithEnv->Parse($VatFormula);
+    my $ShipExpr = $ArithEnv->Parse($ShippingFormula);
+
+# Evaluate it with the current price many times:
+
+    $ArithEnv->VarSetScalar('Price', $price);
+    $VatTax = $ArithEnv->EvalToScalar($VatExpr);
+    $Shipping = $ArithEnv->EvalToScalar($ShipExpr);
+
+
 =head1 HOW TO USE
 
 An expression needs to be first compiled (parsed) and the resulting tree may be run (evaluated)
@@ -1163,8 +1184,11 @@ The return value should be the variable value.
 =item FuncEval
 
 This will evaluate functions.
-The arguments are: 0 - the value returned by C<new>; 1 - the name of the function
-to be evaluated; 2... - an array of function arguments.
+The arguments are:
+0 - the value returned by C<new>;
+1 - the tree as returned by C<ParseString>;
+2 - the name of the function to be evaluated;
+3... - an array of function arguments.
 This should return the value of the function: scalar or array.
 
 The purpose is to permit different functions than those provided (eg C<abs()>) to be made available.
@@ -1175,9 +1199,14 @@ your own evaluator.
 
 If defined this will be called when evaluating functions.
 If a defined value is returned that value is used in the expression, it should be numeric or string.
+If this returns C<undef> the name of function will be matched against the built in functions.
 This is called before the standard functions are tested and thus can redefine the built in functions.
 The arguments are as C<FuncEval>.
 
+New function names must be added to property C<Functions>:
+
+  $ArithEnv->{Functions}->{someFunc} = 1;
+
 =item RoundNegatives
 
 See the description of the C<round> function.
@@ -1266,7 +1295,7 @@ The arguments are: 0 - the value returned by C<new>; 1 - tree to evaluate.
 =head2 Functions that may be used in expressions
 
 The following functions may be used in expressions, if you want more than this write your own
-function evaluator and set it with method C<SetOpt>;
+function evaluator and set 'ExtraFuncEval' with method C<SetOpt>;
 The POSIX package is used to provide some of the functions.
 
 =over 4
@@ -1339,6 +1368,35 @@ Eg the following will return 1:
 
 =back
 
+Example of user defined functions:
+
+    # Function that provides extra functions - ie user functions
+    # sumArgs	numeric sum of arguments
+    # A user defined function must return a scalar or list; it MUST not return undef.
+    sub moreFunctions {
+    	my ($self, $tree, $fname, @arglist) = @_;
+    
+    	if($fname eq 'sumArgs') {
+    		my $sum = 0;
+    		$sum += $_ for @arglist;
+    		return $sum;
+    	}
+    
+    	# Return undef so that in built functions are scanned
+    	return undef;
+    }
+    
+    # MUST put user defined functions here so that it is known as a function - while parsing:
+    $ArithEnv->{Functions}->{sumArgs} = 1;
+    
+    $ArithEnv->SetOpt(ExtraFuncEval => \&moreFunctions);
+
+Used in an expression thus:
+
+    sum := sumArgs(2, 4, 6, 8)
+    list : = (12, 13, 21, 9, -3)
+    sum := sumArgs(list)
+
 =head2 Variables
 
 Variables can two forms, there is no difference in usage between any of them.
@@ -1364,6 +1422,7 @@ In strings surrounded by double quotes the following escapes will be recognised:
 	\t	tab
 	\\	\
 	\xXX	character with hex value XX. Eg: \x0A \x3b
+	\u{XXX}	unicode character. Eg: \u{20AC} is the Euro
 
 A backslash followed by anything else is left as is - ie the backslash will remain.
 
@@ -1371,8 +1430,8 @@ A backslash followed by anything else is left as is - ie the backslash will rema
 
 The operators should not surprise any Perl/C programmer, with the exception that assignemnt
 is C<:=>. Operators associate left to right except for C<:=> which associates right to left.
-Precedence may be overridden with parenthesis C<( )>@.
-<> is the same as C<!=>.
+Precedence may be overridden with parenthesis C<( )>.
+C<E<lt>E<gt>> is the same as C<!=>.
 
 	++ --	Pre increment/decrement only
 	+ - ~ !	(Monadic)
@@ -1452,6 +1511,10 @@ You can create a variable by setting index C<0>.
 
 Index greater than C<ArrayMaxIndex> cannot be used to assign to an array. See method C<SetOpt>.
 
+Assigning C<()> is the same as assigning C<EmptyList>, eg:
+
+	em := ()
+
 =head2 Conditions and loops
 
 Conditional assignment B<used> to be done by use of the ternary operator, but no longer:
@@ -1536,9 +1599,9 @@ Alain D D Williams <addw@phcomp.co.uk>
 
 =head2 Copyright and Version
 
-Version "1.45", this is available as: $Math::Expression::Version.
+Version "1.47", this is available as: $Math::Expression::Version.
 
-Copyright (c) 2003, 2015 Parliament Hill Computers Ltd/Alain D D Williams. All rights reserved.
+Copyright (c) 2003, 2016 Parliament Hill Computers Ltd/Alain D D Williams. All rights reserved.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. Please see the module source
 for the full copyright.
